# Interface::Validation Mixin Directive Role
package Interface::Validation::DirectiveRole::Mixin;

use Bubblegum;
use Function::Parameters;
use Moose::Role;

our $VERSION = '0.01_01'; # VERSION

requires 'mixin';

1;
