# Interface::Validation Filter Directive Role
package Interface::Validation::DirectiveRole::Filter;

use Bubblegum;
use Function::Parameters;
use Moose::Role;

our $VERSION = '0.01_01'; # VERSION

1;
