# Interface::Validation Control Directive Name
package Interface::Validation::Directive::Name;

use Bubblegum;
use Function::Parameters;
use Moose;

our $VERSION = '0.01_01'; # VERSION

extends 'Interface::Validation::Directive';

1;
